// Public Domain

#include <stdio.h>
#include <errno.h>
#include <fs_attr.h>
#include <Application.h>
#include <Directory.h>
#include <Entry.h>
#include <File.h>
#include <Node.h>
#include <Path.h>
#include <String.h>
#include "RenameEmail.h"

int main() 
{
	RenameEmail	*	app	=	new RenameEmail();

	app->Run();

	return 0;
}

RenameEmail::RenameEmail() : BApplication("application/x-vnd.kirilla.rename-email")
{
}

void RenameEmail::RefsReceived(BMessage * message)
{
	entry_ref	ref;
	int32		index	=	0;
	
	while (message->FindRef("refs", index++, & ref) == B_OK)
		Rename (ref);
}

void RenameEmail::ReadyToRun (void)
{
	PostMessage(B_QUIT_REQUESTED);
}

void RenameEmail::Rename(entry_ref ref)
{
	BEntry	entry;
	
	if (entry.SetTo(&ref) != B_OK)
	{
		perror("entry.SetTo()");
		return;
	}

	if(entry.IsDirectory())
	{
		BDirectory dir;
		if (dir.SetTo(&entry) != B_OK)
		{
			perror("dir.SetTo()");
			return;
		}
		
		entry_ref sub_ref;
		while(dir.GetNextRef(&sub_ref) == B_OK)
			Rename(sub_ref);		
	}
	else if(entry.IsFile())
	{
		BFile file;
		if (file.SetTo(& entry, B_READ_ONLY) != B_OK)
		{
			perror("file.SetTo()");
			return;
		}
		
		RenameThis	(& entry);
		
	}
	else if (entry.IsSymLink())
	{
		fprintf(stderr, "Entry is symbolic link. Ignoring it.\n");
	}
	else
	{
		BString message	=	"Entry is neither file, folder nor symlink:\n";
		BPath	path(&entry);
		message.Append(path.Path());
		
		fprintf(stderr, "%s\n", message.String());
	}
}

void
RenameEmail::RenameThis (BEntry * entry)
{
	BNode node;
	attr_info info;
	char * attribute = new char [5];
	char * timestring = new char [50];
	BString filename;
	
	if (node.SetTo(entry) != B_OK)
	{
		perror("node.SetTo()");
		return;
	}
	
	if (node.GetAttrInfo("MAIL:when", & info) != B_OK)
	{
		perror("node.GetAttrInfo()");
		return;
	}
	
	if (node.ReadAttr("MAIL:when", B_TIME_TYPE, 0, attribute, 4) != 4)
	{
		perror("node.ReadAttr()");
		return;
	}
	
	strftime(timestring, 49, "%Y%m%d_%H%M%S", localtime((const time_t *) attribute));
	filename = timestring;
	filename += ".eml";
	int32 counter = 1;
	
	while (entry->Rename(filename.String()) == B_FILE_EXISTS)
	{
		filename = timestring;
		filename << "_";
		filename << counter;
		filename << ".eml";

		++counter;
	}
	
	if (errno)
	{
		perror("entry->Rename()");
		return;
	}
}
